
// ======================================================= draw galvanic skin response
void drawGsr() {
  
  int yPos = 15;                                        // vertical display position
  int ySize = 80;                                       // vertical size of graph
  int frameBottom = 110;                                // draw frame limits
  int frameTop = 0;

  if (displayPulse) {                                   // move graph to upper screen half if pulse is also displayed
    yPos = 5;                                           // vertical display position
    ySize = 40;                                         // vertical size of graph
    frameBottom = 50;                                   // drawing area limits
    frameTop = 0;
  }

  int lineEnd = 0;
  int lineBegin = 0;
  for (int i = 0; i < 160; i++) {                                        // cycle through all columns
    tft.drawFastVLine(i - 1, -2, frameBottom + 3, TFT_BLACK);            // black backround lines
    if ((i + loopCount) % 10 == 0) {
      tft.drawPixel(i - 1, int(ySize + yPos), color(0, .9, .9));         // draw range lines in background
      tft.drawPixel(i - 1, int(yPos), color(0, .9, .9));
    }

    lineBegin = constrain(((1 - ((gsrMOhms[159 - i] - gsrMin) / gsrRange)) * ySize) + yPos, frameTop, frameBottom); // calculate line height
    if (i == 0)lineEnd = lineBegin;
    uint16_t graphColor = TFT_CYAN;
    if (lineEnd == frameTop && lineBegin == frameTop && (i + loopCount) % 3 > 0) graphColor = TFT_BLACK;    // draw graph as dotted line when hitting limits
    if (lineEnd == frameBottom && lineBegin == frameBottom && (i + loopCount) % 3 > 0) graphColor = TFT_BLACK;
    tft.drawLine( i - 1, lineBegin, i - 1 , lineEnd, graphColor);
    lineEnd = lineBegin;
  }


  // ----------------------------------------------------- write gsr text
  int shiftY = 0;
  if (displayPulse) shiftY = -60;

  tft.setCursor(1, 115 + shiftY);
  tft.setTextColor(color(0, .9, .9));
  tft.print("RANGE: ");
  if (gsrValueUpdate) {                                      // only updated once every half second
    tft.fillRect(43, 115 + shiftY, 30, 8, TFT_BLACK);
    tft.print(gsrRangeDisplay);
  }

  tft.setCursor(77, 115 + shiftY);
  tft.setTextColor(TFT_CYAN);
  tft.print("MOhm: ");
  if (gsrMOhms[0] < 30) {                                 // if fingers are on sensor
    if (gsrValueUpdate) {                                    // only updated once every half second
      tft.fillRect(112, 115 + shiftY, 30, 8,  TFT_BLACK);
      tft.print(gsrAverage);
    }
  } else {                                                // if no resistance is detected
    tft.fillRect(112, 115 + shiftY, 30, 8,  TFT_BLACK);
    tft.setTextColor(TFT_CYAN);
    tft.print("-----");
  }
  // ----------------------------------------------------- draw gsr arrows
  if (gsrValueUpdate) {                                                // only updated once every half second
    tft.fillRect(147, 115 + shiftY, 9, 8,  TFT_BLACK);
    pGsrArrow = gsrArrow;
    gsrArrow = 0;

    if ((gsrAverage - gsrOldAverage) > .03 && gsrMOhms[0] < 30) {   // if gsr value has increased by more than 30KOhm:
      drawArrowUp(color(0.5, .8, .8), shiftY);                        // draw arrow up
      gsrArrow = 1;
    }
    if ((gsrAverage - gsrOldAverage) < -.03 && gsrMOhms[0] < 30) {  // if gsr value has decreased by more than 30KOhm:
      drawArrowDown(color(0.5, 1, 1), shiftY);                      // draw arrow down
      gsrArrow = -1;
    }
  }
}

// ======================================================= draw pulse

void drawPulse() {

  // ----------------------------------------------------- draw pulse curve
  int yPos = 68;                                        // vertical display position
  int ySize = 40;                                       // vertical size of graph
  int graphBegin, graphEnd;
  for (int i = 0; i < 79; i++) {                        // cycle through all columns
    tft.drawFastVLine(i * 2 - 1, 64, 52, TFT_BLACK);    // black backround line
    tft.drawFastVLine(i * 2 , 64, 52, TFT_BLACK);       // black backround line
    tft.drawPixel(i * 2, 16 + yPos, color(.9, .1, .1)); // draw threshold
    graphBegin = constrain(map(pulseSamples[79 - i], pulseMin, pulseMax, ySize, 0), 0, ySize) + yPos;
    if (i == 0)graphEnd = graphBegin;
    tft.drawLine(i * 2, graphBegin, i * 2 - 1 , graphEnd, TFT_RED);
    graphEnd = graphBegin;
  }

  // ----------------------------------------------------- write heartbeat text
  tft.setCursor(1, 118);
  tft.setTextColor(TFT_RED);
  tft.print("HEART RATE: ");
  if (heartBeat && !pHeartBeat) {
    tft.fillRect(70, 117, 25, 8, TFT_BLACK);
    if (bpm > 25 && bpm < 220) {
      tft.print(int(bpm + .5));
    } else {
      tft.print("---");
    }
  }

  // ----------------------------------------------------- draw heart (pulse detection)
  uint16_t heartColor = color(.5, .5, .5);
  if (heartBeat)heartColor = TFT_RED;
  drawHeart(112, 113, heartColor);

  // ----------------------------------------------------- draw heart beat arrows

  pHeartArrow = heartArrow;
  heartArrow = 0;

  if (heartBeat && !pHeartBeat) {
    tft.fillRect(145, 117, 12, 9, TFT_BLACK);
    if ((int(bpm) - int(oldBpm)) > 2) {
      drawArrowUp(TFT_RED, 3);
      heartArrow = 1;
    }
    if ((int(oldBpm) - int(bpm)) > 2) {
      drawArrowDown(color(.95, 0, 0), 3);
      heartArrow = -1;
    }
  }
}


// ======================================================= draw heart
void drawHeart(int x, int y, uint16_t c) {
  tft.drawFastHLine(2 + x, 6 + y, 2, c);
  tft.drawFastHLine(5 + x, 6 + y, 2, c);
  tft.drawFastHLine(1 + x, 7 + y, 7, c);
  tft.drawFastHLine(1 + x, 8 + y, 7, c);
  tft.drawFastHLine(2 + x, 9 + y, 5, c);
  tft.drawFastHLine(3 + x, 10 + y, 3, c);
  tft.drawPixel(4 + x, 11 + y, c);
}

// ======================================================= draw arrow functions
void drawArrowUp(uint16_t c, int y) {
  for (int i = 1; i < 6; i++) {
    tft.drawFastHLine(152 - i, 114 + i + y, i * 2 - 1, c);
  }
  tft.drawFastHLine(150, 120 + y, 3, c);
  tft.drawFastHLine(150, 121 + y, 3, c);
  tft.drawFastHLine(150, 122 + y, 3, c);
}

void drawArrowDown(uint16_t c, int y) {
  for (int i = 1; i < 6; i++) {
    tft.drawFastHLine(152 - (6 - i), 117 + i + y, (6 - i) * 2 - 1, c);
  }
  tft.drawFastHLine(150, 115 + y, 3, c);
  tft.drawFastHLine(150, 116 + y, 3, c);
  tft.drawFastHLine(150, 117 + y, 3, c);
}
// ======================================================= convert rgb to gfx-library-style 2-byte color value

uint16_t color(float r, float g, float b) {
  r = r * 31;
  g = g * 63;
  b = b * 31;

  uint8_t ir = constrain(r, 0, 31);
  uint8_t ig = constrain(g, 0, 63);
  uint8_t ib = constrain(b, 0, 31);

  uint16_t returnColor = (ir & 0b00011111) << 11 | (ig & 0b00111111) << 5 | (ib & 0b00011111);
  return returnColor;
}
