void readGsr() {
  // ----------------------------------------------------- read galvanic skin response
  float gsrOversampling = 0;
  for (int i = 0; i < 20; i++) {                        // oversampling input 20x
    gsrOversampling += analogRead(GSRinputPin);
  }
  gsrOversampling = gsrOversampling / 20.0;
  float voltage2 = gsrOversampling / 1023.0 * 5.0;      // voltage drop at skin (R2)
  if (voltage2 == 0) voltage2 = .0001;                  // avoid subsequent division by zero
  float  voltage1 = 5.0 - voltage2;                     // voltage drop at R1 (5.6MOhm)
  gsrMOhms[0] = voltage1 * 5.6 / voltage2;              // skin resistance in MegaOhms
  if (gsrMOhms[0] > 30) gsrMOhms[0] = 30;               // constrain value to 30 MOhms
  float newGsrMin = 1000.0;                             // initialize min and max values
  float newGsrMax = 0.0;
  for (int i = 159; i > 0; i--) {                       // move gsr ring buffer
    gsrMOhms[i] = gsrMOhms[i - 1];
    if (i < 20) {                                       // for the newest 20 samplings:
      if (gsrMOhms[i] > newGsrMax) newGsrMax = gsrMOhms[i];     // save max gsr value
      if (gsrMOhms[i] < newGsrMin) newGsrMin = gsrMOhms[i];     // save min gsr value
    }
  }
  gsrMin = (gsrMin * 30 + newGsrMin) / 31;              // filtering min & max
  gsrMax = (gsrMax * 30 + newGsrMax) / 31;
  gsrRange   = ((gsrMax + .05) - (gsrMin - .05));       // range of gsr in MOhms

  // ----------------------------------------------------- update values once every half second
  gsrValueUpdate = false;
  if (millis() > gsrUpdate) {
    gsrUpdate = millis() + 500L;
    gsrOldAverage = gsrAverage;                         // store old average value for comparison
    gsrAverage = 0;
    for (int i = 0; i < 10; i++) {                      // gsr average
      gsrAverage += gsrMOhms[i];
    }
    gsrAverage /= 10;
    gsrRangeDisplay = gsrRange;                         // gsr range
    gsrValueUpdate = true;
  }
}
